package cd.casic.module.machine.Interceptor;

import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.http.HttpStatus;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.web.socket.WebSocketHandler;
import org.springframework.web.socket.server.HandshakeInterceptor;

import java.net.URI;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.Map;

import static cd.casic.framework.commons.exception.util.ServiceExceptionUtil.exception;
import static cd.casic.module.machine.contants.MachineErrorCodeConstants.*;

@Component
@Slf4j
public class WebSocketHandshakeInterceptor implements HandshakeInterceptor {
    @Override
    public boolean beforeHandshake(
        @NotNull ServerHttpRequest request,
        @NotNull ServerHttpResponse response,
        @NotNull WebSocketHandler wsHandler,
        @NotNull Map<String, Object> attributes
    ) {
        log.info("WebSocket握手请求: {}", request.getURI());

        // 从URL参数中获取id
        String id = extractIdFromUrl(request.getURI());

        System.out.println("-----------------------------------------");
        if (id != null) {
            attributes.put("machineId", id); // 将id存入attributes
            log.info("握手验证成功 - ID: {}", id);
            return true;
        } else {
            response.setStatusCode(HttpStatus.BAD_REQUEST);
            log.warn("握手验证失败 - 缺少 id URL 参数");
            return false;
        }
    }

    @Override
    public void afterHandshake(
        @NotNull ServerHttpRequest request,
        @NotNull ServerHttpResponse response,
        @NotNull WebSocketHandler wsHandler,
        Exception exception
    ) {
        if (exception == null) {
            log.info("WebSocket握手完成 - URI: {}",
                request.getURI());
        } else {
            log.error("WebSocket握手异常 - URI: {}, 异常信息: {}",
                request.getURI(),
                exception.getMessage(),
                exception);
        }
    }

    // 从URI中提取id参数的辅助方法
    private String extractIdFromUrl(URI uri) {
        try {
            String query = uri.getQuery();
            if (query != null) {
                String[] params = query.split("&");
                for (String param : params) {
                    String[] keyValue = param.split("=");
                    if (keyValue.length == 2 && "id".equalsIgnoreCase(keyValue[0])) { // 修改为匹配id
                        return URLDecoder.decode(keyValue[1], StandardCharsets.UTF_8);
                    }
                }
            }
        } catch (Exception e) {
            log.error("解析URL参数失败", e);
            throw exception(FAILED_TO_PARSE_URL_PARAMETERS);
        }
        return null;
    }

}
