package cd.casic.module.machine.component;

import org.springframework.stereotype.Component;
import org.springframework.web.socket.WebSocketSession;

import java.util.Collection;
import java.util.concurrent.ConcurrentHashMap;

@Component("machineWebSocketSessionManger")
//管理webSocketSession
public class WebSocketSessionManager {

    //webSocketSessionId - WebSocketSession 保存 WebSocketSession 对象与会话 ID 的映射
    private static final ConcurrentHashMap<String, WebSocketSession> WebSocketSessionMap = new ConcurrentHashMap<>();

    //webSocketSessionId - WebSocketConnection 与远程机器的会话管理
    private static final ConcurrentHashMap<String, WebSocketConnection> sessionConnectionMap = new ConcurrentHashMap<>();

    //机器id - WebSocketConnection
    private static final ConcurrentHashMap<Long, WebSocketConnection> webSocketSessionConnectionMap = new ConcurrentHashMap<>();

    public static void addWebSocketSession(String sessionId, WebSocketSession session) {
        WebSocketSessionMap.put(sessionId, session);
    }

    /**
     * 获取 WebSocketSession
     */
    public static WebSocketSession getWebSocketSession(String sessionId) {
        return WebSocketSessionMap.get(sessionId);
    }

    /**
     * 移除 WebSocketSession
     */
    public static void removeWebSocketSession(String sessionId) {
        WebSocketSessionMap.remove(sessionId);
    }

    /**
     * 检查 sessionId 是否存在
     */
    public static boolean containsWebSocketSession(String sessionId) {
        return WebSocketSessionMap.containsKey(sessionId);
    }

    /**
     * 获取所有 WebSocketSession
     */
    public static Collection<WebSocketSession> getAllWebSocketSessions() {
        return WebSocketSessionMap.values();
    }

    public static void addWebSocketConnection(String sessionId, WebSocketConnection connection) {
        sessionConnectionMap.put(sessionId, connection);
    }

    /**
     * 获取 WebSocketConnection
     */
    public static WebSocketConnection getWebSocketConnection(String sessionId) {
        return sessionConnectionMap.get(sessionId);
    }

    /**
     * 移除 WebSocketConnection
     */
    public static void removeWebSocketConnection(String sessionId) {
        sessionConnectionMap.remove(sessionId);
    }

    /**
     * 检查 sessionId 是否存在
     */
    public static boolean containsWebSocketConnection(String sessionId) {
        return sessionConnectionMap.containsKey(sessionId);
    }

    /**
     * 获取所有 WebSocketConnection
     */
    public static ConcurrentHashMap<Long, WebSocketConnection> getAllWebSocketConnections() {
        return webSocketSessionConnectionMap;
    }

    /**
     * 添加 WebSocketConnection
     */
    public static void addWebSocketConnectionByMachineId(Long machineId, WebSocketConnection connection) {
        webSocketSessionConnectionMap.put(machineId, connection);
    }

    /**
     * 获取 WebSocketConnection
     */
    public static WebSocketConnection getWebSocketConnectionByMachineId(Long machineId) {
        return webSocketSessionConnectionMap.get(machineId);
    }

    /**
     * 移除 WebSocketConnection
     */
    public static void removeWebSocketConnectionByMachineId(Long machineId) {
        webSocketSessionConnectionMap.remove(machineId);
    }

    /**
     * 检查 machineId 是否存在
     */
    public static boolean containsMachineId(Long machineId) {
        return webSocketSessionConnectionMap.containsKey(machineId);
    }

}
