package cd.casic.module.machine.handler;

import cd.casic.module.machine.component.WebSocketConnection;
import cd.casic.module.machine.component.WebSocketSessionManager;
import cd.casic.module.machine.enums.ConnectionStatus;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Component;
import org.springframework.web.socket.CloseStatus;
import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketSession;
import org.springframework.web.socket.handler.TextWebSocketHandler;

import java.io.IOException;

import static cd.casic.framework.commons.exception.util.ServiceExceptionUtil.exception;
import static cd.casic.module.machine.contants.MachineErrorCodeConstants.*;

//WebSocket处理器
@Component("machineWebSocketHandler")
public class MachineWebSocketHandler extends TextWebSocketHandler {

    @Override
    public void afterConnectionEstablished(@NotNull WebSocketSession webSocketSession) {
        Long machineId = (Long) webSocketSession.getAttributes().get("machineId");
        //保存webSocketSession信息
        WebSocketSessionManager.addWebSocketSession(webSocketSession.getId(), webSocketSession);
        WebSocketSessionManager.addWebSocketConnection(webSocketSession.getId(), WebSocketSessionManager.getWebSocketConnectionByMachineId(machineId));
        try {
            webSocketSession.sendMessage(new TextMessage("欢迎连接到 WebSocket 服务器!"));
        } catch (IOException e) {
            throw exception(WEBSOCKET_SEND_MESSAGE_ERROR);
        }
    }

    // 处理文本消息
    @Override
    protected void handleTextMessage(WebSocketSession webSocketSession, TextMessage message) {
        String payload = message.getPayload();
        String sessionId = webSocketSession.getId();
        // 从管理器获取连接
        WebSocketConnection webSocketConnection = WebSocketSessionManager.getWebSocketConnection(sessionId);

        if (webSocketConnection != null && ConnectionStatus.CONNECTING.equals(webSocketConnection.getConnectionStatus())) {
            // 转发消息到远程机器
            webSocketConnection.executeCommand(webSocketSession, payload);
        } else if (webSocketConnection != null) {
            webSocketConnection.sendErrorMessage(webSocketSession, "连接已断开，无法发送消息");
        } else {
            throw exception(WEBSOCKET_SEND_MESSAGE_ERROR);
        }
    }

    @Override
    public void afterConnectionClosed(WebSocketSession webSocketSession, @NotNull CloseStatus status) {
        String sessionId = webSocketSession.getId();
        // 获取并关闭相关的 WebSocketConnection
        WebSocketConnection webSocketConnection = WebSocketSessionManager.getWebSocketConnection(sessionId);
        if (webSocketConnection != null) {
            webSocketConnection.disconnect();
        }
        // 从管理器中移除会话和连接
        WebSocketSessionManager.removeWebSocketSession(sessionId);
        WebSocketSessionManager.removeWebSocketConnection(sessionId);
        Long machineInfoId = (Long) webSocketSession.getAttributes().get("machineId");
        WebSocketSessionManager.removeWebSocketConnectionByMachineId(machineInfoId);
    }
}
