package cd.casic.module.machine.service;

import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.module.machine.controller.vo.MachineInfoVO;
import cd.casic.module.machine.dal.dataobject.MachineInfoDO;
import cd.casic.module.machine.enums.ConnectionStatus;
import jakarta.validation.Valid;

import java.util.List;
import java.util.Map;

public interface MachineInfoService {
    /**
     * 新增机器
     * @return 新增机器的id
     */
    Long createMachine(MachineInfoVO MachineInfoVO);

    /**
     * 查看机器列表
     * @return 分院
     */
    PageResult<MachineInfoDO> listMachineInfo(@Valid MachineInfoVO MachineInfoVO);

    void updateMachineInfo(@Valid MachineInfoVO machineInfoVO);

    Integer updateStatus(@Valid MachineInfoVO machineInfoVO);

    void bindingSecretKey(List<Long> machineInfoIds, Long secretKeyId);

    void deleteMachineInfoList(String machineInfoIds);

    void deleteMachineInfo(Long machineInfoId);


    /**
     * 测试机器连接
     *
     * @param id 机器id
     * @return 连接是否成功
     */
    boolean testConnection(Long id);

    /**
     * 连接远程机器
     *
     * @param id 机器id
     * @return  连接后端文件树
     */
    Map<String, Object> connect(Long id);

    /**
     * 获取机器连接状态
     *
     * @return 连接状态
     */
    ConnectionStatus getConnectionStatus(Long id);

    /**
     * 获取所有连接状态
     *
     * @return 机器名称到连接状态的映射
     */
    Map<Long, ConnectionStatus> getAllConnectionStatus();

//    /**
//     * 上传文件到远程机器
//     *
//     * @param sessionId      会话ID
//     * @param localFilePath  本地文件路径
//     * @param remoteFilePath 远程文件路径
//     * @return 操作结果
//     */
//    boolean uploadFile(String sessionId, String localFilePath, String remoteFilePath);
//
//    /**
//     * 从远程机器下载文件
//     *
//     * @param sessionId      会话ID
//     * @param remoteFilePath 远程文件路径
//     * @param localFilePath  本地文件路径
//     * @return 操作结果
//     */
//    boolean downloadFile(String sessionId, String remoteFilePath, String localFilePath);

    /**
     * 校验机器是否存在
     *
     * @param id 机器id
     */
    MachineInfoDO validateMachineInfoExists(Long id);

    /**
     * 根据路径获得远程文件树
     *
     * @param machineId 机器id
     * @param path 文件夹路径
     * @return 远程文件树
     */
    Map<String, Object> fileTreeNode(Long machineId, String path);
}
