package cd.casic.module.process.task.artifact.service.impl;

import cd.casic.ci.commons.bean.exception.ApplicationException;
import cd.casic.ci.commons.bean.process.setting.AuthThird;
import cd.casic.module.process.process.setting.AuthThirdService;
import cd.casic.module.process.support.util.util.PipelineRequestUtil;
import cd.casic.module.process.task.artifact.model.XpackRepository;
import cd.casic.module.process.task.artifact.model.XpackRepositoryQuery;
import cd.casic.module.process.task.artifact.service.TaskArtifactXpackService;
import cd.casic.module.process.task.code.service.impl.TaskCodeGittokServiceImpl;
import jakarta.annotation.Resource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;

import java.util.HashMap;
import java.util.List;
import java.util.Objects;

/**
 * @ClassName TaskArtifactXpackServiceImpl
 * @Author hopeli
 * @Date 2025/5/2 19:19
 * @Version 1.0
 */
@Service
public class TaskArtifactXpackServiceImpl implements TaskArtifactXpackService {

    private static final Logger logger = LoggerFactory.getLogger(TaskCodeGittokServiceImpl.class);

    @Resource
    AuthThirdService authThirdService;

    @Resource
    PipelineRequestUtil requestUtil;


    @Override
    public List<XpackRepository> findAllRepository(String authId){
        AuthThird authServer = authThirdService.findOneAuthServer(authId);

        if (Objects.isNull(authServer)){
            return null;
        }

        String serverAddress = authServer.getServerAddress();
        XpackRepositoryQuery repositoryQuery = new XpackRepositoryQuery();
        repositoryQuery.setRepositoryType("local");
        try {
            HttpHeaders headers = requestUtil.initHeaders(MediaType.APPLICATION_JSON, new HashMap<>());
            String requestUrl = serverAddress+"/api/xpackRepository/findRepositoryList";
            return requestUtil.requestPostList(headers, requestUrl, repositoryQuery, XpackRepository.class);

        } catch (Throwable throwable){
            String message = throwable.getMessage();
            logger.error(message);
            if (message.contains("未订阅")){
                throw new ApplicationException("当前企业未订阅GitPuk!");
            }
            if (throwable instanceof ApplicationException){
                throw new ApplicationException(message);
            }
            throw new ApplicationException("无法连接到："+serverAddress);
        }
    }

    public XpackRepository findRepository(String authId,String rpyId){
        AuthThird authServer = authThirdService.findOneAuthServer(authId);
        if (Objects.isNull(authServer) || Objects.isNull(rpyId)){
            return null;
        }
        String serverAddress = authServer.getServerAddress();

        try {
            HttpHeaders headers = requestUtil.initHeaders(MediaType.APPLICATION_JSON, new HashMap<>());
            String requestUrl = serverAddress+"/api/xpackRepository/findRepository";
            MultiValueMap<String, String> multiValueMap = new LinkedMultiValueMap<>();
            multiValueMap.add("id",rpyId);
            return requestUtil.requestPost(headers, requestUrl, multiValueMap, XpackRepository.class);

        } catch (Throwable throwable){
            String message = throwable.getMessage();
            logger.error(message);
            if (throwable instanceof ApplicationException){
                throw new ApplicationException(message);
            }
            if (message.contains("未订阅")){
                throw new ApplicationException("当前企业未订阅GitPuk!");
            }
            throw new ApplicationException("无法连接到："+serverAddress);
        }
    }
}
